@@ -144,6 +144,15 @@ class Trainer(BaseTrainer):
             return self.train_dataloader
 
         assert self.schema is not None, "schema is required to generate Train Dataloader"
+
+        # Set global_rank and global_size if DDP is used
+        if self.args.local_rank != -1:
+            local_rank = self.args.local_rank
+            global_size = self.args.world_size
+        else:
+            local_rank = None
+            global_size = None
+
         return T4RecDataLoader.parse(self.args.data_loader_engine).from_schema(
             self.schema,
             self.train_dataset_or_path,
@@ -152,6 +161,8 @@ class Trainer(BaseTrainer):
             drop_last=self.args.dataloader_drop_last,
             shuffle=True,
             shuffle_buffer_size=self.args.shuffle_buffer_size,
+            global_rank=local_rank,
+            global_size=global_size,
         )
 
     def get_eval_dataloader(self, eval_dataset=None):
@@ -610,11 +621,6 @@ class Trainer(BaseTrainer):
         """
         import os
 
-        try:
-            import cloudpickle
-        except ImportError:
-            cloudpickle = None
-
         logger.info("Saving model...")
         output_dir = os.path.join(
             self.args.output_dir, f"{PREFIX_CHECKPOINT_DIR}-{self.state.global_step}"
@@ -625,11 +631,7 @@ class Trainer(BaseTrainer):
         # save the serialized model
         if save_model_class:
             # TODO : fix serialization of DatasetSchema object
-            if cloudpickle is None:
-                raise ValueError("cloudpickle is required to save model class")
-
-            with open(os.path.join(output_dir, "model_class.pkl"), "wb") as out:
-                cloudpickle.dump(self.model.wrapper_module, out)
+            self.model.wrapper_module.save(output_dir)
 
     def load_model_trainer_states_from_checkpoint(self, checkpoint_path, model=None):
         """
@@ -653,7 +655,9 @@ class Trainer(BaseTrainer):
             except ImportError:
                 raise ImportError("cloudpickle is required to load model class")
             logger.info("Loading model class")
-            model = cloudpickle.load(open(os.path.join(checkpoint_path, "model_class.pkl"), "rb"))
+            model = cloudpickle.load(
+                open(os.path.join(checkpoint_path, "t4rec_model_class.pkl"), "rb")
+            )
 
         self.model = HFWrapper(model)
         logger.info("Loading weights of previously trained model")
